{
  // constants

  const hotkey = "F7"
  const containerClassName = "language-monitor-container"
  const returnedTextMaxLength = 70
  const maxEntries = 30
  const isSDKversion = process.versions["nw-flavor"] === "sdk"
  const systemAndPluginSuffix = isSDKversion ? ", content: (see console)" : ""

  const createContainer = () => {
    const container = document.createElement("div")
    container.classList.add(containerClassName)
    return container
  }

  const createStyleTag = () => {
    const styleTag = document.createElement("style")
    styleTag.textContent = `
      .${containerClassName} {
        display: none;
        flex-direction: column;
        color: white;
      
        box-sizing: border-box;
        padding: 1rem;
        position: absolute;
        top: 0;
        width: 100vw;
        height: 100vh;
        background-color: rgba(0, 0, 0, 0.8);

        /* important because for some reason, z-index gets set to 0 on the inline style of the div */
        z-index: 10 !important;
      }

      .${containerClassName} > header {
        display: flex;
        margin-bottom: 1rem;
        font-family: sans-serif;
      }

      .${containerClassName} > header > span {
        margin-left: .5rem;
      }

      .${containerClassName} > header > h3 {
        flex: 1;
        margin: 0;
      }

      .${containerClassName} > main {
        display: flex;
        flex-direction: column;
        overflow: hidden;
      }

      .${containerClassName} code {
        margin-top: .5rem;
      }
    `
    return styleTag
  }

  const createHeader = () => {
    const header = document.createElement("header")
    header.innerHTML = `
      <h3>Language Calls Monitor</h3>
      <span>Press ${hotkey} to toggle this overlay</span>
    `
    return header
  }

  /**
   * @param {HTMLElement} container
   * @param {string} text
   */
  const addEntry = (container, text) => {
    const entry = document.createElement("code")
    entry.innerText = text
    container.prepend(entry)

    while (container.childElementCount > maxEntries) {
      container.removeChild(container.lastElementChild)
    }
  }
  
  /** @param {HTMLElement} container */
  const addKeyboardListener = container => {
    document.addEventListener("keydown", event => {
      if (event.key !== hotkey) {
        return
      }
      
      const isVisible = container.style.display === "flex"
      container.style.display = isVisible ? "none" : "flex"
    })
  }

  /**
   * @param {string} sharedOutput
   * @param {any} result
   */
  const conditionallyLog = (sharedOutput, result) => {
    if (!isSDKversion) {
      return
    }

    console.log(`[LCM Mod] ${sharedOutput}, content:`, result)
  }

  const origGetTextData = LanguageManager.getTextData.bind(LanguageManager)
  const origGetPluginText = LanguageManager.getPluginText.bind(LanguageManager)
  const origGetSystemText = LanguageManager.getSystemText.bind(LanguageManager)

  /**
   * @param {string} text
   * @param {number} max
   */
  const getLimitedText = (text, max) => {
    const maybeEllipses = text.length > max ? " (...)" : ""
    return text.slice(0, max) + maybeEllipses
  }

  // main

  let id = 0

  document.head.appendChild(createStyleTag())

  const container = createContainer()
  container.appendChild(createHeader())
  const main = document.createElement("main")
  container.appendChild(main)
  document.body.appendChild(container)

  addKeyboardListener(container)

  const getPrefix = () => {
    if (!isSDKversion) {
      return ""
    }

    return `#${++id} `
  }

  LanguageManager.getTextData = (file, name, language) => {
    const result = origGetTextData(file, name, language)
    const sharedOutput = `${getPrefix()}text | file: ${file}, name: ${name}`
    const textToShow = typeof result.text === "string" ? result.text : "[[.text property is missing / not a string!]]"
    addEntry(main, `${sharedOutput}\n> "${getLimitedText(textToShow, returnedTextMaxLength)}"`)
    conditionallyLog(sharedOutput, result)
    return result
  }

  LanguageManager.getPluginText = (type, name, language = LanguageManager._language) => {
    const result = origGetPluginText(type, name, language)
    const sharedOutput = `${getPrefix()}plugin | type: ${type}, name: ${name}`
    addEntry(main, sharedOutput + systemAndPluginSuffix)
    conditionallyLog(sharedOutput, result)
    return result
  }

  LanguageManager.getSystemText = (type, name, language) => {
    const result = origGetSystemText(type, name, language)
    const sharedOutput = `${getPrefix()}system | type: ${type}, name: ${name}`
    addEntry(main, sharedOutput + systemAndPluginSuffix)
    conditionallyLog(sharedOutput, result)
    return result
  }
}